@@ -22,6 +22,8 @@
   ##
   ## 1. setting arguments
   ##
+  if(missing(geodata))
+    geodata <- list(coords = coords, data = data)
   require(mva)
   call.fc <- match.call()
   n.cov.pars <- obj.likfit$npars - length(obj.likfit$beta)
@@ -132,7 +134,7 @@
   ##
   ## 2. data preparation
   ##
-  trend <- trend.spatial(trend=obj.likfit$trend, coords=coords)
+  trend <- trend.spatial(trend=obj.likfit$trend, geodata = geodata)
   data <- as.vector(data)
   dimnames(trend) <- list(NULL, NULL)
   if(obj.likfit$transform.info$fix.lambda == TRUE) {
